local _, addon = ...
local cfg = addon.config
local lib = CreateFrame('Frame')
addon.lib = lib

local E, F, C = unpack(_G['lyn'])

-- ----------------------------------------------------------------------------------------------------

local _, playerClass = UnitClass('player')
local floor, fmod = floor, math.fmod
local day, hour, minute = 86400, 3600, 60
local bnot, band = bit.bnot, bit.band

local flags = {}
for i=0, 7 do
	flags[i] = bit.lshift(1, i)
end

local methods = {
	Hide = function(self)
		if(not self.__visible) then return end

		for i=1,8 do
			self[i]:Hide()
		end

		self.__visible = nil
	end,

	Show = function(self)
		if(self.__visible) then return end

		for i=1,8 do
			self[i]:Show()
		end

		self.__visible = true
	end,

	SetColor = function(self, r, g, b, a)
		if(r == self.__r and g == self.__g and b == self.__b and a == self.__a) then
			return
		end

		for i=1,8 do
			self[i]:SetVertexColor(r, g, b, a)
		end

		self.__r, self.__g, self.__b, self.__a = r, g, b, a
	end,

	SetParent = function(self, parent)
		self.__parent = parent
	end,

	SetVisible = function(self, left, right, top, bottom)
		local mask = 0xff
		if(not left) then
			mask = mask - flags[0]
			mask = band(mask, bnot(flags[1]))
			mask = band(mask, bnot(flags[7]))
		end

		if(not right) then
			mask = mask - flags[4]
			mask = band(mask, bnot(flags[3]))
			mask = band(mask, bnot(flags[5]))
		end

		if(not top) then
			mask = mask - flags[2]
			mask = band(mask, bnot(flags[1]))
			mask = band(mask, bnot(flags[3]))
		end

		if(not bottom) then
			mask = mask - flags[6]
			mask = band(mask, bnot(flags[7]))
			mask = band(mask, bnot(flags[5]))
		end

		for i=0, 7 do
			if(band(mask, flags[i]) ~= 0) then
				self[i + 1]:Show()
			else
				self[i + 1]:Hide()
			end
		end
	end,

	SetPoint = function(self, point)
		point = point or self.__parent

		for i=1, 8 do
			self[i]:ClearAllPoints()
		end

		local Left = self[1]
		Left:SetPoint('RIGHT', point, 'LEFT')
		Left:SetPoint('TOP')
		Left:SetPoint('BOTTOM')
		Left:SetWidth(16)

		local TopLeft = self[2]
		TopLeft:SetPoint('BOTTOMRIGHT', point, 'TOPLEFT')
		TopLeft:SetSize(16, 16)

		local Top = self[3]
		Top:SetPoint('BOTTOM', point, 'TOP')
		Top:SetPoint('LEFT')
		Top:SetPoint('RIGHT')
		Top:SetHeight(16)

		local TopRight = self[4]
		TopRight:SetPoint('BOTTOMLEFT', point, 'TOPRIGHT')
		TopRight:SetSize(16, 16)

		local Right = self[5]
		Right:SetPoint('LEFT', point, 'RIGHT')
		Right:SetPoint('TOP')
		Right:SetPoint('BOTTOM')
		Right:SetWidth(16)

		local BottomRight = self[6]
		BottomRight:SetPoint('TOPLEFT', point, 'BOTTOMRIGHT')
		BottomRight:SetSize(16, 16)

		local Bottom = self[7]
		Bottom:SetPoint('TOP', point, 'BOTTOM')
		Bottom:SetPoint('LEFT')
		Bottom:SetPoint('RIGHT')
		Bottom:SetHeight(16)

		local BottomLeft = self[8]
		BottomLeft:SetPoint('TOPRIGHT', point, 'BOTTOMLEFT')
		BottomLeft:SetSize(16, 16)
	end
}
methods.__index = methods

lib.CreateBorder = function(self, texture)
	local Border = setmetatable({
		__visible = true,
	}, methods)

	for i=1,8 do
		local T = self:CreateTexture(nil, 'BORDER')
		T:SetTexture(texture)
		Border[i] = T
	end

	local Left = Border[1]
	Left:SetTexCoord(0/8, 1/8, 0/8, 8/8)

	local TopLeft = Border[2]
	TopLeft:SetTexCoord(6/8, 7/8, 8/8, 0/8)

	-- Affine transofrmation matrix used
	-- {{cos(90), sin(90), 1}, {-sin(90), cos(90), 1}, {0, 0, 1}} *
	-- {{32, 0,  -11}, {0, 1, 0}, {0, 0, 1}}
	local Top = Border[3]
	Top:SetTexCoord(.5, -.75, .375, -.75, .5, -.625, .375, -.625)

	local TopRight = Border[4]
	TopRight:SetTexCoord(7/8, 6/8, 8/8, 0/8)

	local Right = Border[5]
	Right:SetTexCoord(1/8, 0/8, 0/8, 8/8)

	local BottomRight = Border[6]
	BottomRight:SetTexCoord(5/8, 6/8, 8/8, 0/8)

	-- Affine transofrmation matrix used
	-- {{cos(90), -sin(90), 1}, {sin(90), cos(90), 0}, {0, 0, 1}} *
	-- {{8, 0,  -3}, {0, 1, 0}, {0, 0, 1}}
	local Bottom = Border[7]
	Bottom:SetTexCoord(.375, 1, .5, 1, .375, 0, .5, 0)

	local BottomLeft = Border[8]
	BottomLeft:SetTexCoord(6/8, 5/8, 8/8, 0/8)

	Border:SetParent(self)
	Border:SetPoint()

	self.Border = Border
	return Border
end

lib.shortenName = function(string)
	return string:gsub("(%w)[^%s]+%s", "%1. ")
end


lib.utf8sub = function(string, i, dots)
	local bytes = string:len()
	if bytes <= i then
		return string
	else
		local len, pos = 0, 1
		while pos <= bytes do
			len = len + 1
			local c = string:byte(pos)
			if c > 0 and c <= 127 then
				pos = pos + 1
			elseif c >= 194 and c <= 223 then
				pos = pos + 2
			elseif c >= 224 and c <= 239 then
				pos = pos + 3
			elseif c >= 240 and c <= 244 then
				pos = pos + 4
			end
			if len == i then break end
		end
		if len == i and pos <= bytes then
			return string:sub(1, pos - 1)..(dots and '...' or '')
		else
			return string
		end
	end
end

lib.formatValue = function(v)
	if v > 1E10 then
		return (floor(v/1E9)).."b"
	elseif v > 1E9 then
		return (floor((v/1E9)*10)/10).."b"
	elseif v > 1E7 then
		return (floor(v/1E6)).."m"
	elseif v > 1E6 then
		return (floor((v/1E6)*10)/10).."m"
	elseif v > 1E4 then
		return (floor(v/1E3)).."k"
	elseif v > 1E3 then
		return (floor((v/1E3)*10)/10).."k"
	else
		return v
	end
end

lib.hex = function(r, g, b)
	if r then
		if (type(r) == 'table') then
			if(r.r) then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
		end
		return ('|cff%02x%02x%02x'):format(r * 255, g * 255, b * 255)
	end
end

local formatTime = function(s)
	if s >= day then
		return format('%dd', floor(s/day + 0.5))
	elseif s >= hour then
		return format('%dh', floor(s/hour + 0.5))
	elseif s >= minute then
		return format('%dm', floor(s/minute + 0.5))
	end
	return format('%d', fmod(s, minute))
end

lib.CreateBackdrop = function(self, alpha, inset)
	local f = self
	if self:GetObjectType() == 'Texture' then
		f = self:GetParent()
	end

	f:SetBackdrop({
		bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
		tiled = false,
		insets = {left = inset or -1, right = inset or -1, top = inset or -1, bottom = inset or -1}
	})
	f:SetBackdropColor(0, 0, 0, 1)

	local t = f:CreateTexture(nil, "BACKGROUND")
	t:SetColorTexture(0, 0, 0, alpha or .5)
	t:SetPoint('TOPLEFT', f, -3, 3)
	t:SetPoint('BOTTOMRIGHT', f, 3, -3)
end

lib.CreateFrameBorder = function(self, inset)
	F:CreateBorder(self, inset or -3)
end

lib.CreateBackground = function(self, alpha, inset, sub)
	local f = self:CreateTexture(nil, "BACKGROUND", nil, sub or 1)
	f:SetColorTexture(0, 0, 0, alpha or 1)
	if inset ~= nil then
		f:SetPoint('TOPLEFT', self, -inset, inset)
		f:SetPoint('BOTTOMRIGHT', self, inset, -inset)
	else
		f:SetAllPoints(self)
	end
end

lib.PostUpdateHealth = function(health, unit, min, max)
	if UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) then
		health:SetValue(0)
	end

	local percent = math.floor((min / max) * 100+0.5)
	if unit ~= 'target' and ((unit == 'player' or unit:sub(1, 5) == 'party') and percent < 75) then
		local r, g, b = oUF.ColorGradient(min, max, 1,0,0, 1,1,0, 0,1,0)
		health:SetStatusBarColor(r, g, b)
	end
end

lib.PostUpdatePower = function(power, unit, min, max)
	if(
		min == 0 or max == 0 or not UnitIsConnected(unit) or
		UnitIsDead(unit) or UnitIsGhost(unit) or unit:sub(1, 4) == 'boss'
	) then
		power:Hide()
		power:SetBackdropColor(0, 0, 0, 0)
		power:GetParent().Health:ClearAllPoints()
		power:GetParent().Health:SetPoint('TOPLEFT', power, 0, 0)
		power:GetParent().Health:SetPoint('BOTTOMRIGHT', power, 0, 0)
	else
		power:Show()
		power:SetBackdropColor(0, 0, 0, 1)
		power:GetParent().Health:ClearAllPoints()
		power:GetParent().Health:SetPoint('TOPLEFT', power, 3, -3)
		power:GetParent().Health:SetPoint('BOTTOMRIGHT', power, -3, 3)
	end
end

lib.PostUpdateReadyCheckFade = function(self)
	local Silhouette = self:GetParent():GetParent().Silhouette
	Silhouette:Show()
end

lib.PostUpdateReadyCheck = function(self, status)
		local silhouette = self:GetParent():GetParent().Silhouette
		if self and self:IsShown() then
			silhouette:Hide()
		end
	end

lib.AnimateHover = function(self, unit, enable)
	if not enable then return end
	self.Hover:Show()
	if not pulseGroup then
		local pulseGroup = self.Hover:CreateAnimationGroup(self:GetName()..'pulseGroup')
		pulseGroup:SetLooping'BOUNCE'

		local pulse = pulseGroup:CreateAnimation'Alpha'
		pulse:SetDuration(1)
		pulse:SetFromAlpha(.8)
		pulse:SetToAlpha(0)
		pulse:SetSmoothing'OUT'
	end
	_G[self:GetName()..'pulseGroup']:Play()
end

local ProxyOverlay = function(overlay, ...)
	local r, g, b  = ...
	overlay:GetParent():SetBorderColor(r, g, b, 1)
end

local HideOverlay = function(overlay)
	overlay:GetParent():SetBorderColor(unpack(C.BORDER_COLOR_DARK))
end

local PostCreateIcon = function(Auras, button)

	local count = button.count
	count:ClearAllPoints()
	count:SetFont(cfg.font.big, 12, 'OUTLINE')
	count:SetPoint('BOTTOMRIGHT', button, -1, 1)

	button.icon:SetTexCoord(.07, .93, .07, .93)

	-- XXX: Actually make a glow texture of auras.
	lib.CreateBorder(button, cfg.path .. 'glow')
	button.Border:SetColor(0, 0, 0, 1)

	F:CreateBorder(button, -1)
	button:SetBorderColor(unpack(C.BORDER_COLOR_DARK))

	button.duration = button:CreateFontString(nil, 'OVERLAY')
	button.duration:SetPoint('TOP', button, 0, -2)
	button.duration:SetFont(cfg.font.big, 14, 'OUTLINE')
	button.duration:SetTextColor(1, 0.82, 0)
	button.duration:SetShadowOffset(1, -1)
	button.duration:SetShadowColor(0, 0, 0, 1)
	button.duration:SetJustifyH('CENTER')

	local overlay = button.overlay
	overlay.SetVertexColor = ProxyOverlay
	overlay:Hide()
	overlay.Show = overlay.Hide
	overlay.Hide = HideOverlay
end


lib.CreateAura = function(self, num, rows, size)

	local Auras = CreateFrame('Frame', nil, self)

	Auras:SetSize((num * (size + 9))/rows, (size + 9) * rows)
	Auras.num = num
	Auras.size = size
	Auras.spacing = 6

	Auras.disableCooldown = true
	Auras.PostCreateIcon = PostCreateIcon


	return Auras
end

local AuraTimer_OnUpdate = function(icon, elapsed)
	if icon.timeLeft then
		icon.timeLeft = math.max(icon.timeLeft - elapsed, 0)

		if icon.timeLeft > 0 and icon.timeLeft < 60 then
			icon.duration:SetFormattedText(formatTime(icon.timeLeft))

			if (icon.timeLeft < 6) then
				icon.duration:SetTextColor(0.69, 0.31, 0.31)
			else
				icon.duration:SetTextColor(1, 0.82, 0)
			end

		else
			icon.duration:SetText()
		end
	end
end

lib.PostUpdateIcon = function(icons, unit, icon, index, offset, filter, isDebuff)
	local name, _, count, dtype, duration, expirationTime = UnitAura(unit, index, icon.filter)

	if duration and duration > 0 then
		icon.timeLeft = expirationTime - GetTime()
	else
		icon.timeLeft = math.huge
	end

--	icon:SetScript('OnUpdate', function(self, elapsed)
--		auras:AuraTimer_OnUpdate(self, elapsed)
	
	if icon.isDebuff and unit == 'target' and cfg.onlyPlayerDebuffs == false then
		if icon.isPlayer then
			icon.icon:SetDesaturated(false)
			icon.count:SetAlpha(1)
		else
			icon.Border:SetColor(0, 0, 0, 1)
			icon.icon:SetDesaturated(true)
			icon.timeLeft = math.huge
			icon.count:SetAlpha(0)
		end
	end

	icon:SetScript('OnUpdate', function(self, elapsed)
		AuraTimer_OnUpdate(self, elapsed)
	end)
end

local IsPlayer = { player = true, pet = true, vehicle = true }

lib.PlayerCustomFilter = function(self, unit, iconFrame, name, icon, count, debuffType, duration, expirationTime, caster, canStealOrPurge, _, spellID, canApplyAura, isBossAura, casterIsPlayer, nameplateShowAll, timeMod)
	if cfg.playerAuras[spellID] --[[ and IsPlayer[caster] ]] then
		 return true
	end
	return false
end

lib.PartyCustomFilter = function(self, unit, iconFrame, name, icon, count, debuffType, duration, expirationTime, caster, canStealOrPurge, _, spellID, canApplyAura, isBossAura, casterIsPlayer, nameplateShowAll, timeMod)
	if cfg.raidAuras[spellID] and IsPlayer[caster] then
		 return true
	end
	return false
end

local CastbarCheckShield = function(self, unit)
	local color
	if self.notInterruptible and UnitCanAttack("player", unit) then
		color = cfg.color.castbar.shield
	elseif unit ~= 'target' then
		color = cfg.color.castbar.player
	else
		color = cfg.color.castbar.target
	end
	self:SetStatusBarColor(color.r, color.g, color.b, 1)
end

local CastbarCheckCast = function(bar, unit, name, castID)
	CastbarCheckShield(bar, unit)
end

local CastbarCheckChannel = function(bar, unit, name)
	CastbarCheckShield(bar, unit)
end

local CastbarCustomTimeText = function(self, duration)
	self.Time:SetFormattedText("%.1f", self.max - duration)
end

lib.CreateCastbar = function(self, unit)
	local castbar = CreateFrame('StatusBar', 'oUF_LynCastbar' .. unit, self)

	if unit == 'target' then
		local height = 24

		castbar:SetStatusBarTexture(cfg.statusbar.normal)
		castbar:SetHeight(height)
		castbar:SetWidth(300)
		castbar:SetPoint(unpack(cfg.castbar))

		lib.CreateBackground(castbar)
		
		F:CreateBorder(castbar, -1)
		castbar:SetBorderColor(unpack(C.BORDER_COLOR_DARK))

		local spark = castbar:CreateTexture(nil,'OVERLAY')
		spark:SetBlendMode('ADD')
		spark:SetSize(height, height * 2.3)
		castbar.Spark = spark

		local text = castbar:CreateFontString(nil, "OVERLAY")
		text:SetFont(cfg.font.big, 14)
		text:SetShadowColor(0,0,0,0.8)
		text:SetShadowOffset(1,-1)
		text:SetPoint('LEFT', castbar, 'LEFT', 5, 0)
		text:SetJustifyH('LEFT')
		castbar.Text = text

		local ctime = castbar:CreateFontString(nil, "OVERLAY")
		ctime:SetFont(cfg.font.big, 14)
		ctime:SetShadowColor(0,0,0,0.8)
		ctime:SetShadowOffset(1, -1)
		ctime:SetPoint('RIGHT', castbar, 'RIGHT', -5, 0)
		ctime:SetJustifyH('RIGHT')
		castbar.Time = ctime
	elseif unit == 'player' then
		castbar:SetStatusBarTexture(cfg.statusbar.castbar)
		castbar:SetAllPoints(self.Health)
		castbar:SetFrameLevel(self.Health:GetFrameLevel() + 1)
		castbar:SetAlpha(1)
		
		local spark = castbar:CreateTexture(nil,'OVERLAY')
		spark:SetBlendMode('ADD')
		spark:SetSize(24, castbar:GetHeight() * 2.3)
		castbar.Spark = spark

		local text = castbar:CreateFontString(nil, "OVERLAY")
		text:SetFont(cfg.font.big, 12, 'OUTLINE')
		text:SetShadowColor(0,0,0,1)
		text:SetShadowOffset(1,-1)
		text:SetPoint('TOP', self, 'BOTTOM', 0, -5)
		text:SetPoint('RIGHT', 5, 0)
		text:SetJustifyH('RIGHT')
		castbar.Text = text
	elseif unit == 'focus' then
		castbar:SetStatusBarTexture(cfg.statusbar.castbar)
		castbar:SetAllPoints(self.Health)
		castbar:SetFrameLevel(self.Health:GetFrameLevel() + 1)
		castbar:SetAlpha(1)
	end

	self.Castbar = castbar

	self.Castbar.CustomTimeText = CastbarCustomTimeText
	self.Castbar.PostCastStart = CastbarCheckCast
	self.Castbar.PostChannelStart = CastbarCheckChannel
end
